# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, overload
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ..._base_client import make_request_options
from ...types.models import chat_completion_create_params
from ...types.parameter_bindings_param import ParameterBindingsParam
from ...types.shared.completion_response import CompletionResponse
from ...types.evaluation_datasets.flexible_message_param import FlexibleMessageParam

__all__ = ["ChatCompletionsResource", "AsyncChatCompletionsResource"]


class ChatCompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ChatCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ChatCompletionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionResponse:
        """
        ### Description

        Interact with the LLM model using the specified model_deployment_id. You can
        include a list of messages as the conversation history. The conversation can
        feature multiple messages from the roles user, assistant, and system. If the
        chosen model does not support chat completion, the API will revert to simple
        completion, disregarding the provided history. The endpoint manages context
        length exceedance optimistically: it estimates the token count from the provided
        history and prompt, and if it exceeds the context or approaches 80% of it, the
        exact token count will be calculated, and the history will be trimmed to fit the
        context.

        ```json
        {
          "prompt": "Generate 5 more",
          "chat_history": [
            {
              "role": "system",
              "content": "You are a name generator. Do not generate anything else than names"
            },
            { "role": "user", "content": "Generate 5 names" },
            {
              "role": "assistant",
              "content": "1. Olivia Bennett\n2. Ethan Carter\n3. Sophia Ramirez\n4. Liam Thompson\n5. Ava Mitchell"
            }
          ]
        }
        ```

        Args:
          chat_history: Chat history entries with roles and messages. If there's no history, pass an
              empty list.

          prompt: New user prompt. This will be sent to the model with a user role.

          frequency_penalty: Penalize tokens based on how much they have already appeared in the text.
              Positive values encourage the model to generate new tokens and negative values
              encourage the model to repeat tokens. Available for models provided by LLM
              Engine and OpenAI.

          logprobs: Whether to return logprobs. Currently only supported for llmengine chat models.

          max_tokens: The maximum number of tokens to generate in the completion. The token count of
              your prompt plus max_tokens cannot exceed the model's context length. If not,
              specified, max_tokens will be determined based on the model used: | Model API
              family | Model API default | EGP applied default | | --- | --- | --- | | OpenAI
              Completions |
              [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
              | `context window - prompt size` | | OpenAI Chat Completions |
              [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
              | `context window - prompt size` | | LLM Engine |
              [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
              parameter is required | `100` | | Anthropic Claude 2 |
              [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
              parameter is required | `10000` |

          presence_penalty: Penalize tokens based on if they have already appeared in the text. Positive
              values encourage the model to generate new tokens and negative values encourage
              the model to repeat tokens. Available for models provided by LLM Engine and
              OpenAI.

          stop_sequences: List of up to 4 sequences where the API will stop generating further tokens. The
              returned text will not contain the stop sequence.

          stream: Flag indicating whether to stream the completion response

          temperature: What sampling temperature to use, between [0, 2]. Higher values like 1.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic. Setting temperature=0.0 will enable fully
              deterministic (greedy) sampling.NOTE: The temperature parameter range for some
              model is limited to [0, 1] if the given value is above the available range, it
              defaults to the max value.

          top_k: Sample from the k most likely next tokens at each step. Lower k focuses on
              higher probability tokens. Available for models provided by Google and LLM
              Engine.

          top_logprobs: Number of top logprobs to return. Currently only supported for llmengine chat
              models.

          top_p: The cumulative probability cutoff for token selection. Lower values mean
              sampling from a smaller, more top-weighted nucleus. Available for models
              provided by Google, LLM Engine, and OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        stream: Literal[True],
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Stream[CompletionResponse]:
        """
        ### Description

        Interact with the LLM model using the specified model_deployment_id. You can
        include a list of messages as the conversation history. The conversation can
        feature multiple messages from the roles user, assistant, and system. If the
        chosen model does not support chat completion, the API will revert to simple
        completion, disregarding the provided history. The endpoint manages context
        length exceedance optimistically: it estimates the token count from the provided
        history and prompt, and if it exceeds the context or approaches 80% of it, the
        exact token count will be calculated, and the history will be trimmed to fit the
        context.

        ```json
        {
          "prompt": "Generate 5 more",
          "chat_history": [
            {
              "role": "system",
              "content": "You are a name generator. Do not generate anything else than names"
            },
            { "role": "user", "content": "Generate 5 names" },
            {
              "role": "assistant",
              "content": "1. Olivia Bennett\n2. Ethan Carter\n3. Sophia Ramirez\n4. Liam Thompson\n5. Ava Mitchell"
            }
          ]
        }
        ```

        Args:
          chat_history: Chat history entries with roles and messages. If there's no history, pass an
              empty list.

          prompt: New user prompt. This will be sent to the model with a user role.

          stream: Flag indicating whether to stream the completion response

          frequency_penalty: Penalize tokens based on how much they have already appeared in the text.
              Positive values encourage the model to generate new tokens and negative values
              encourage the model to repeat tokens. Available for models provided by LLM
              Engine and OpenAI.

          logprobs: Whether to return logprobs. Currently only supported for llmengine chat models.

          max_tokens: The maximum number of tokens to generate in the completion. The token count of
              your prompt plus max_tokens cannot exceed the model's context length. If not,
              specified, max_tokens will be determined based on the model used: | Model API
              family | Model API default | EGP applied default | | --- | --- | --- | | OpenAI
              Completions |
              [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
              | `context window - prompt size` | | OpenAI Chat Completions |
              [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
              | `context window - prompt size` | | LLM Engine |
              [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
              parameter is required | `100` | | Anthropic Claude 2 |
              [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
              parameter is required | `10000` |

          presence_penalty: Penalize tokens based on if they have already appeared in the text. Positive
              values encourage the model to generate new tokens and negative values encourage
              the model to repeat tokens. Available for models provided by LLM Engine and
              OpenAI.

          stop_sequences: List of up to 4 sequences where the API will stop generating further tokens. The
              returned text will not contain the stop sequence.

          temperature: What sampling temperature to use, between [0, 2]. Higher values like 1.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic. Setting temperature=0.0 will enable fully
              deterministic (greedy) sampling.NOTE: The temperature parameter range for some
              model is limited to [0, 1] if the given value is above the available range, it
              defaults to the max value.

          top_k: Sample from the k most likely next tokens at each step. Lower k focuses on
              higher probability tokens. Available for models provided by Google and LLM
              Engine.

          top_logprobs: Number of top logprobs to return. Currently only supported for llmengine chat
              models.

          top_p: The cumulative probability cutoff for token selection. Lower values mean
              sampling from a smaller, more top-weighted nucleus. Available for models
              provided by Google, LLM Engine, and OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        stream: bool,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionResponse | Stream[CompletionResponse]:
        """
        ### Description

        Interact with the LLM model using the specified model_deployment_id. You can
        include a list of messages as the conversation history. The conversation can
        feature multiple messages from the roles user, assistant, and system. If the
        chosen model does not support chat completion, the API will revert to simple
        completion, disregarding the provided history. The endpoint manages context
        length exceedance optimistically: it estimates the token count from the provided
        history and prompt, and if it exceeds the context or approaches 80% of it, the
        exact token count will be calculated, and the history will be trimmed to fit the
        context.

        ```json
        {
          "prompt": "Generate 5 more",
          "chat_history": [
            {
              "role": "system",
              "content": "You are a name generator. Do not generate anything else than names"
            },
            { "role": "user", "content": "Generate 5 names" },
            {
              "role": "assistant",
              "content": "1. Olivia Bennett\n2. Ethan Carter\n3. Sophia Ramirez\n4. Liam Thompson\n5. Ava Mitchell"
            }
          ]
        }
        ```

        Args:
          chat_history: Chat history entries with roles and messages. If there's no history, pass an
              empty list.

          prompt: New user prompt. This will be sent to the model with a user role.

          stream: Flag indicating whether to stream the completion response

          frequency_penalty: Penalize tokens based on how much they have already appeared in the text.
              Positive values encourage the model to generate new tokens and negative values
              encourage the model to repeat tokens. Available for models provided by LLM
              Engine and OpenAI.

          logprobs: Whether to return logprobs. Currently only supported for llmengine chat models.

          max_tokens: The maximum number of tokens to generate in the completion. The token count of
              your prompt plus max_tokens cannot exceed the model's context length. If not,
              specified, max_tokens will be determined based on the model used: | Model API
              family | Model API default | EGP applied default | | --- | --- | --- | | OpenAI
              Completions |
              [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
              | `context window - prompt size` | | OpenAI Chat Completions |
              [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
              | `context window - prompt size` | | LLM Engine |
              [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
              parameter is required | `100` | | Anthropic Claude 2 |
              [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
              parameter is required | `10000` |

          presence_penalty: Penalize tokens based on if they have already appeared in the text. Positive
              values encourage the model to generate new tokens and negative values encourage
              the model to repeat tokens. Available for models provided by LLM Engine and
              OpenAI.

          stop_sequences: List of up to 4 sequences where the API will stop generating further tokens. The
              returned text will not contain the stop sequence.

          temperature: What sampling temperature to use, between [0, 2]. Higher values like 1.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic. Setting temperature=0.0 will enable fully
              deterministic (greedy) sampling.NOTE: The temperature parameter range for some
              model is limited to [0, 1] if the given value is above the available range, it
              defaults to the max value.

          top_k: Sample from the k most likely next tokens at each step. Lower k focuses on
              higher probability tokens. Available for models provided by Google and LLM
              Engine.

          top_logprobs: Number of top logprobs to return. Currently only supported for llmengine chat
              models.

          top_p: The cumulative probability cutoff for token selection. Lower values mean
              sampling from a smaller, more top-weighted nucleus. Available for models
              provided by Google, LLM Engine, and OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["chat_history", "prompt"], ["chat_history", "prompt", "stream"])
    def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        stream: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionResponse | Stream[CompletionResponse]:
        if not model_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `model_deployment_id` but received {model_deployment_id!r}"
            )
        return self._post(
            f"/v4/models/{model_deployment_id}/chat-completions",
            body=maybe_transform(
                {
                    "chat_history": chat_history,
                    "prompt": prompt,
                    "frequency_penalty": frequency_penalty,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "model_request_parameters": model_request_parameters,
                    "presence_penalty": presence_penalty,
                    "stop_sequences": stop_sequences,
                    "stream": stream,
                    "temperature": temperature,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                },
                chat_completion_create_params.ChatCompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompletionResponse,
            stream=stream or False,
            stream_cls=Stream[CompletionResponse],
        )


class AsyncChatCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncChatCompletionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionResponse:
        """
        ### Description

        Interact with the LLM model using the specified model_deployment_id. You can
        include a list of messages as the conversation history. The conversation can
        feature multiple messages from the roles user, assistant, and system. If the
        chosen model does not support chat completion, the API will revert to simple
        completion, disregarding the provided history. The endpoint manages context
        length exceedance optimistically: it estimates the token count from the provided
        history and prompt, and if it exceeds the context or approaches 80% of it, the
        exact token count will be calculated, and the history will be trimmed to fit the
        context.

        ```json
        {
          "prompt": "Generate 5 more",
          "chat_history": [
            {
              "role": "system",
              "content": "You are a name generator. Do not generate anything else than names"
            },
            { "role": "user", "content": "Generate 5 names" },
            {
              "role": "assistant",
              "content": "1. Olivia Bennett\n2. Ethan Carter\n3. Sophia Ramirez\n4. Liam Thompson\n5. Ava Mitchell"
            }
          ]
        }
        ```

        Args:
          chat_history: Chat history entries with roles and messages. If there's no history, pass an
              empty list.

          prompt: New user prompt. This will be sent to the model with a user role.

          frequency_penalty: Penalize tokens based on how much they have already appeared in the text.
              Positive values encourage the model to generate new tokens and negative values
              encourage the model to repeat tokens. Available for models provided by LLM
              Engine and OpenAI.

          logprobs: Whether to return logprobs. Currently only supported for llmengine chat models.

          max_tokens: The maximum number of tokens to generate in the completion. The token count of
              your prompt plus max_tokens cannot exceed the model's context length. If not,
              specified, max_tokens will be determined based on the model used: | Model API
              family | Model API default | EGP applied default | | --- | --- | --- | | OpenAI
              Completions |
              [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
              | `context window - prompt size` | | OpenAI Chat Completions |
              [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
              | `context window - prompt size` | | LLM Engine |
              [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
              parameter is required | `100` | | Anthropic Claude 2 |
              [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
              parameter is required | `10000` |

          presence_penalty: Penalize tokens based on if they have already appeared in the text. Positive
              values encourage the model to generate new tokens and negative values encourage
              the model to repeat tokens. Available for models provided by LLM Engine and
              OpenAI.

          stop_sequences: List of up to 4 sequences where the API will stop generating further tokens. The
              returned text will not contain the stop sequence.

          stream: Flag indicating whether to stream the completion response

          temperature: What sampling temperature to use, between [0, 2]. Higher values like 1.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic. Setting temperature=0.0 will enable fully
              deterministic (greedy) sampling.NOTE: The temperature parameter range for some
              model is limited to [0, 1] if the given value is above the available range, it
              defaults to the max value.

          top_k: Sample from the k most likely next tokens at each step. Lower k focuses on
              higher probability tokens. Available for models provided by Google and LLM
              Engine.

          top_logprobs: Number of top logprobs to return. Currently only supported for llmengine chat
              models.

          top_p: The cumulative probability cutoff for token selection. Lower values mean
              sampling from a smaller, more top-weighted nucleus. Available for models
              provided by Google, LLM Engine, and OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        stream: Literal[True],
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncStream[CompletionResponse]:
        """
        ### Description

        Interact with the LLM model using the specified model_deployment_id. You can
        include a list of messages as the conversation history. The conversation can
        feature multiple messages from the roles user, assistant, and system. If the
        chosen model does not support chat completion, the API will revert to simple
        completion, disregarding the provided history. The endpoint manages context
        length exceedance optimistically: it estimates the token count from the provided
        history and prompt, and if it exceeds the context or approaches 80% of it, the
        exact token count will be calculated, and the history will be trimmed to fit the
        context.

        ```json
        {
          "prompt": "Generate 5 more",
          "chat_history": [
            {
              "role": "system",
              "content": "You are a name generator. Do not generate anything else than names"
            },
            { "role": "user", "content": "Generate 5 names" },
            {
              "role": "assistant",
              "content": "1. Olivia Bennett\n2. Ethan Carter\n3. Sophia Ramirez\n4. Liam Thompson\n5. Ava Mitchell"
            }
          ]
        }
        ```

        Args:
          chat_history: Chat history entries with roles and messages. If there's no history, pass an
              empty list.

          prompt: New user prompt. This will be sent to the model with a user role.

          stream: Flag indicating whether to stream the completion response

          frequency_penalty: Penalize tokens based on how much they have already appeared in the text.
              Positive values encourage the model to generate new tokens and negative values
              encourage the model to repeat tokens. Available for models provided by LLM
              Engine and OpenAI.

          logprobs: Whether to return logprobs. Currently only supported for llmengine chat models.

          max_tokens: The maximum number of tokens to generate in the completion. The token count of
              your prompt plus max_tokens cannot exceed the model's context length. If not,
              specified, max_tokens will be determined based on the model used: | Model API
              family | Model API default | EGP applied default | | --- | --- | --- | | OpenAI
              Completions |
              [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
              | `context window - prompt size` | | OpenAI Chat Completions |
              [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
              | `context window - prompt size` | | LLM Engine |
              [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
              parameter is required | `100` | | Anthropic Claude 2 |
              [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
              parameter is required | `10000` |

          presence_penalty: Penalize tokens based on if they have already appeared in the text. Positive
              values encourage the model to generate new tokens and negative values encourage
              the model to repeat tokens. Available for models provided by LLM Engine and
              OpenAI.

          stop_sequences: List of up to 4 sequences where the API will stop generating further tokens. The
              returned text will not contain the stop sequence.

          temperature: What sampling temperature to use, between [0, 2]. Higher values like 1.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic. Setting temperature=0.0 will enable fully
              deterministic (greedy) sampling.NOTE: The temperature parameter range for some
              model is limited to [0, 1] if the given value is above the available range, it
              defaults to the max value.

          top_k: Sample from the k most likely next tokens at each step. Lower k focuses on
              higher probability tokens. Available for models provided by Google and LLM
              Engine.

          top_logprobs: Number of top logprobs to return. Currently only supported for llmengine chat
              models.

          top_p: The cumulative probability cutoff for token selection. Lower values mean
              sampling from a smaller, more top-weighted nucleus. Available for models
              provided by Google, LLM Engine, and OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        stream: bool,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionResponse | AsyncStream[CompletionResponse]:
        """
        ### Description

        Interact with the LLM model using the specified model_deployment_id. You can
        include a list of messages as the conversation history. The conversation can
        feature multiple messages from the roles user, assistant, and system. If the
        chosen model does not support chat completion, the API will revert to simple
        completion, disregarding the provided history. The endpoint manages context
        length exceedance optimistically: it estimates the token count from the provided
        history and prompt, and if it exceeds the context or approaches 80% of it, the
        exact token count will be calculated, and the history will be trimmed to fit the
        context.

        ```json
        {
          "prompt": "Generate 5 more",
          "chat_history": [
            {
              "role": "system",
              "content": "You are a name generator. Do not generate anything else than names"
            },
            { "role": "user", "content": "Generate 5 names" },
            {
              "role": "assistant",
              "content": "1. Olivia Bennett\n2. Ethan Carter\n3. Sophia Ramirez\n4. Liam Thompson\n5. Ava Mitchell"
            }
          ]
        }
        ```

        Args:
          chat_history: Chat history entries with roles and messages. If there's no history, pass an
              empty list.

          prompt: New user prompt. This will be sent to the model with a user role.

          stream: Flag indicating whether to stream the completion response

          frequency_penalty: Penalize tokens based on how much they have already appeared in the text.
              Positive values encourage the model to generate new tokens and negative values
              encourage the model to repeat tokens. Available for models provided by LLM
              Engine and OpenAI.

          logprobs: Whether to return logprobs. Currently only supported for llmengine chat models.

          max_tokens: The maximum number of tokens to generate in the completion. The token count of
              your prompt plus max_tokens cannot exceed the model's context length. If not,
              specified, max_tokens will be determined based on the model used: | Model API
              family | Model API default | EGP applied default | | --- | --- | --- | | OpenAI
              Completions |
              [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
              | `context window - prompt size` | | OpenAI Chat Completions |
              [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
              | `context window - prompt size` | | LLM Engine |
              [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
              parameter is required | `100` | | Anthropic Claude 2 |
              [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
              parameter is required | `10000` |

          presence_penalty: Penalize tokens based on if they have already appeared in the text. Positive
              values encourage the model to generate new tokens and negative values encourage
              the model to repeat tokens. Available for models provided by LLM Engine and
              OpenAI.

          stop_sequences: List of up to 4 sequences where the API will stop generating further tokens. The
              returned text will not contain the stop sequence.

          temperature: What sampling temperature to use, between [0, 2]. Higher values like 1.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic. Setting temperature=0.0 will enable fully
              deterministic (greedy) sampling.NOTE: The temperature parameter range for some
              model is limited to [0, 1] if the given value is above the available range, it
              defaults to the max value.

          top_k: Sample from the k most likely next tokens at each step. Lower k focuses on
              higher probability tokens. Available for models provided by Google and LLM
              Engine.

          top_logprobs: Number of top logprobs to return. Currently only supported for llmengine chat
              models.

          top_p: The cumulative probability cutoff for token selection. Lower values mean
              sampling from a smaller, more top-weighted nucleus. Available for models
              provided by Google, LLM Engine, and OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["chat_history", "prompt"], ["chat_history", "prompt", "stream"])
    async def create(
        self,
        model_deployment_id: str,
        *,
        chat_history: Iterable[FlexibleMessageParam],
        prompt: str,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        stop_sequences: List[str] | NotGiven = NOT_GIVEN,
        stream: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_k: float | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompletionResponse | AsyncStream[CompletionResponse]:
        if not model_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `model_deployment_id` but received {model_deployment_id!r}"
            )
        return await self._post(
            f"/v4/models/{model_deployment_id}/chat-completions",
            body=await async_maybe_transform(
                {
                    "chat_history": chat_history,
                    "prompt": prompt,
                    "frequency_penalty": frequency_penalty,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "model_request_parameters": model_request_parameters,
                    "presence_penalty": presence_penalty,
                    "stop_sequences": stop_sequences,
                    "stream": stream,
                    "temperature": temperature,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                },
                chat_completion_create_params.ChatCompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompletionResponse,
            stream=stream or False,
            stream_cls=AsyncStream[CompletionResponse],
        )


class ChatCompletionsResourceWithRawResponse:
    def __init__(self, chat_completions: ChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = to_raw_response_wrapper(
            chat_completions.create,
        )


class AsyncChatCompletionsResourceWithRawResponse:
    def __init__(self, chat_completions: AsyncChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = async_to_raw_response_wrapper(
            chat_completions.create,
        )


class ChatCompletionsResourceWithStreamingResponse:
    def __init__(self, chat_completions: ChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = to_streamed_response_wrapper(
            chat_completions.create,
        )


class AsyncChatCompletionsResourceWithStreamingResponse:
    def __init__(self, chat_completions: AsyncChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = async_to_streamed_response_wrapper(
            chat_completions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "CompletionCreateParamsBase",
    "Image",
    "ModelParameters",
    "CompletionCreateParamsNonStreaming",
    "CompletionCreateParamsStreaming",
]


class CompletionCreateParamsBase(TypedDict, total=False):
    model: Required[
        Union[
            Literal[
                "gpt-4",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0613",
                "gpt-4-vision-preview",
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-16k-0613",
                "gemini-pro",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-preview-0409",
                "gemini-1.5-pro-preview-0514",
                "text-davinci-003",
                "text-davinci-002",
                "text-curie-001",
                "text-babbage-001",
                "text-ada-001",
                "claude-instant-1",
                "claude-instant-1.1",
                "claude-2",
                "claude-2.0",
                "llama-7b",
                "llama-2-7b",
                "llama-2-7b-chat",
                "llama-2-13b",
                "llama-2-13b-chat",
                "llama-2-70b",
                "llama-2-70b-chat",
                "llama-3-8b",
                "llama-3-8b-instruct",
                "llama-3-1-8b-instruct",
                "llama-3-1-70b-instruct",
                "llama-3-70b-instruct",
                "Meta-Llama-3-8B-Instruct-RMU",
                "Meta-Llama-3-8B-Instruct-RR",
                "Meta-Llama-3-8B-Instruct-DERTA",
                "Meta-Llama-3-8B-Instruct-LAT",
                "falcon-7b",
                "falcon-7b-instruct",
                "falcon-40b",
                "falcon-40b-instruct",
                "mpt-7b",
                "mpt-7b-instruct",
                "flan-t5-xxl",
                "mistral-7b",
                "mistral-7b-instruct",
                "mixtral-8x7b",
                "mixtral-8x7b-instruct",
                "mixtral-8x22b-instruct",
                "llm-jp-13b-instruct-full",
                "llm-jp-13b-instruct-full-dolly",
                "zephyr-7b-alpha",
                "zephyr-7b-beta",
                "zephyr-cat-merged",
                "codellama-7b",
                "codellama-7b-instruct",
                "codellama-13b",
                "codellama-13b-instruct",
                "codellama-34b",
                "codellama-34b-instruct",
                "phi-3-mini-4k-instruct",
                "phi-3-cat-merged",
                "dolphin-2.9-llama3-8b",
                "dolphin-2.9-llama3-70b",
            ],
            str,
        ]
    ]
    """The ID of the model to use for completions.

    Users have two options:

    - Option 1: Use one of the supported models from the dropdown.
    - Option 2: Enter the ID of a custom model.

    Note: For custom models we currently only support models finetuned using using
    the Scale-hosted LLM-Engine API.
    """

    prompt: Required[str]
    """Prompt for which to generate the completion.

    Good prompt engineering is crucial to getting performant results from the model.
    If you are having trouble getting the model to perform well, try writing a more
    specific prompt here before trying more expensive techniques such as swapping in
    other models or finetuning the underlying LLM.
    """

    account_id: str
    """The account ID to use for usage tracking. This will be gradually enforced."""

    images: Iterable[Image]
    """List of image urls to be used for image based completions.

    Leave empty for text based completions.
    """

    model_parameters: ModelParameters
    """
    Configuration parameters for the completion model, such as temperature,
    max_tokens, and stop_sequences.

    If not specified, the default value are:

    - temperature: 0.2
    - max_tokens: None (limited by the model's max tokens)
    - stop_sequences: None
    """


class Image(TypedDict, total=False):
    image_url: Required[str]
    """Image URL to run image completion on."""

    detail: str
    """Detail to run image completion with. Defaults to auto"""


class ModelParameters(TypedDict, total=False):
    frequency_penalty: float
    """Penalize tokens based on how much they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models from LLM Engine, and
    OpenAI
    """

    max_tokens: int
    """The maximum number of tokens to generate in the completion.

    The token count of your prompt plus max_tokens cannot exceed the model's context
    length. If not, specified, max_tokens will be determined based on the model
    used: | Model API family | Model API default | EGP applied default | | --- | ---
    | --- | | OpenAI Completions |
    [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
    | `context window - prompt size` | | OpenAI Chat Completions |
    [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
    | `context window - prompt size` | | LLM Engine |
    [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
    parameter is required | `100` | | Anthropic Claude 2 |
    [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
    parameter is required | `10000` |
    """

    presence_penalty: float
    """Penalize tokens based on if they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models from LLM Engine, and
    OpenAI.
    """

    stop_sequences: List[str]
    """List of up to 4 sequences where the API will stop generating further tokens.

    The returned text will not contain the stop sequence.
    """

    temperature: float
    """What sampling temperature to use, between [0, 1].

    Higher values like 0.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic. Setting temperature=0.0 will
    enable fully deterministic (greedy) sampling.
    """

    top_k: Annotated[int, PropertyInfo(alias="topK")]
    """Sample from the k most likely next tokens at each step.

    Lower k focuses on higher probability tokens. Available for models from
    Anthropic, Google, and LLM Engine.
    """

    top_p: Annotated[float, PropertyInfo(alias="topP")]
    """The cumulative probability cutoff for token selection.

    Lower values mean sampling from a smaller, more top-weighted nucleus. Available
    for models from Anthropic, Google, Mistral, LLM Engine, and OpenAI.
    """


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """Whether or not to stream the response.

    Setting this to True will stream the completion in real-time.
    """


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """Whether or not to stream the response.

    Setting this to True will stream the completion in real-time.
    """


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ChunkExtraInfoSchema", "Chunk"]


class Chunk(BaseModel):
    metadata: object

    text: str


class ChunkExtraInfoSchema(BaseModel):
    chunks: List[Chunk]

    schema_type: Optional[Literal["CHUNKS"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp._utils import parse_datetime
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse
from scale_gp.types.knowledge_bases import (
    UploadSchedule,
    UploadScheduleWithViews,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUploadSchedules:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={"strategy": "character"},
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={
                "strategy": "character",
                "chunk_overlap": 0,
                "chunk_size": 1,
                "separator": "separator",
            },
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
            account_id="account_id",
            next_run_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.knowledge_bases.upload_schedules.with_raw_response.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={"strategy": "character"},
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = response.parse()
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.knowledge_bases.upload_schedules.with_streaming_response.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={"strategy": "character"},
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = response.parse()
            assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.create(
                knowledge_base_id="",
                chunking_strategy_config={"strategy": "character"},
                interval=0,
                knowledge_base_data_source_id="knowledge_base_data_source_id",
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
            view=["DataSource"],
        )
        assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.knowledge_bases.upload_schedules.with_raw_response.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = response.parse()
        assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.knowledge_bases.upload_schedules.with_streaming_response.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = response.parse()
            assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.retrieve(
                upload_schedule_id="upload_schedule_id",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `upload_schedule_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.retrieve(
                upload_schedule_id="",
                knowledge_base_id="knowledge_base_id",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={
                "strategy": "character",
                "chunk_overlap": 0,
                "chunk_size": 1,
                "separator": "separator",
            },
            interval=0,
            next_run_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.knowledge_bases.upload_schedules.with_raw_response.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = response.parse()
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.knowledge_bases.upload_schedules.with_streaming_response.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = response.parse()
            assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.update(
                upload_schedule_id="upload_schedule_id",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `upload_schedule_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.update(
                upload_schedule_id="",
                knowledge_base_id="knowledge_base_id",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.list(
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(SyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.list(
            knowledge_base_id="knowledge_base_id",
            account_id="account_id",
            limit=1,
            page=1,
            view=["DataSource"],
        )
        assert_matches_type(SyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.knowledge_bases.upload_schedules.with_raw_response.list(
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = response.parse()
        assert_matches_type(SyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.knowledge_bases.upload_schedules.with_streaming_response.list(
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = response.parse()
            assert_matches_type(SyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.list(
                knowledge_base_id="",
            )

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        upload_schedule = client.knowledge_bases.upload_schedules.delete(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(GenericDeleteResponse, upload_schedule, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.knowledge_bases.upload_schedules.with_raw_response.delete(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = response.parse()
        assert_matches_type(GenericDeleteResponse, upload_schedule, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.knowledge_bases.upload_schedules.with_streaming_response.delete(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = response.parse()
            assert_matches_type(GenericDeleteResponse, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.delete(
                upload_schedule_id="upload_schedule_id",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `upload_schedule_id` but received ''"):
            client.knowledge_bases.upload_schedules.with_raw_response.delete(
                upload_schedule_id="",
                knowledge_base_id="knowledge_base_id",
            )


class TestAsyncUploadSchedules:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={"strategy": "character"},
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={
                "strategy": "character",
                "chunk_overlap": 0,
                "chunk_size": 1,
                "separator": "separator",
            },
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
            account_id="account_id",
            next_run_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.upload_schedules.with_raw_response.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={"strategy": "character"},
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = await response.parse()
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.upload_schedules.with_streaming_response.create(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={"strategy": "character"},
            interval=0,
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = await response.parse()
            assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.create(
                knowledge_base_id="",
                chunking_strategy_config={"strategy": "character"},
                interval=0,
                knowledge_base_data_source_id="knowledge_base_data_source_id",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
            view=["DataSource"],
        )
        assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.upload_schedules.with_raw_response.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = await response.parse()
        assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.upload_schedules.with_streaming_response.retrieve(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = await response.parse()
            assert_matches_type(UploadScheduleWithViews, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.retrieve(
                upload_schedule_id="upload_schedule_id",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `upload_schedule_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.retrieve(
                upload_schedule_id="",
                knowledge_base_id="knowledge_base_id",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config={
                "strategy": "character",
                "chunk_overlap": 0,
                "chunk_size": 1,
                "separator": "separator",
            },
            interval=0,
            next_run_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.upload_schedules.with_raw_response.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = await response.parse()
        assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.upload_schedules.with_streaming_response.update(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = await response.parse()
            assert_matches_type(UploadSchedule, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.update(
                upload_schedule_id="upload_schedule_id",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `upload_schedule_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.update(
                upload_schedule_id="",
                knowledge_base_id="knowledge_base_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.list(
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(AsyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.list(
            knowledge_base_id="knowledge_base_id",
            account_id="account_id",
            limit=1,
            page=1,
            view=["DataSource"],
        )
        assert_matches_type(AsyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.upload_schedules.with_raw_response.list(
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = await response.parse()
        assert_matches_type(AsyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.upload_schedules.with_streaming_response.list(
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = await response.parse()
            assert_matches_type(AsyncPageResponse[UploadScheduleWithViews], upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.list(
                knowledge_base_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        upload_schedule = await async_client.knowledge_bases.upload_schedules.delete(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(GenericDeleteResponse, upload_schedule, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.upload_schedules.with_raw_response.delete(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        upload_schedule = await response.parse()
        assert_matches_type(GenericDeleteResponse, upload_schedule, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.upload_schedules.with_streaming_response.delete(
            upload_schedule_id="upload_schedule_id",
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            upload_schedule = await response.parse()
            assert_matches_type(GenericDeleteResponse, upload_schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.delete(
                upload_schedule_id="upload_schedule_id",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `upload_schedule_id` but received ''"):
            await async_client.knowledge_bases.upload_schedules.with_raw_response.delete(
                upload_schedule_id="",
                knowledge_base_id="knowledge_base_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .model_groups import (
    ModelGroupsResource,
    AsyncModelGroupsResource,
    ModelGroupsResourceWithRawResponse,
    AsyncModelGroupsResourceWithRawResponse,
    ModelGroupsResourceWithStreamingResponse,
    AsyncModelGroupsResourceWithStreamingResponse,
)
from .usage_statistics import (
    UsageStatisticsResource,
    AsyncUsageStatisticsResource,
    UsageStatisticsResourceWithRawResponse,
    AsyncUsageStatisticsResourceWithRawResponse,
    UsageStatisticsResourceWithStreamingResponse,
    AsyncUsageStatisticsResourceWithStreamingResponse,
)

__all__ = [
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "UsageStatisticsResource",
    "AsyncUsageStatisticsResource",
    "UsageStatisticsResourceWithRawResponse",
    "AsyncUsageStatisticsResourceWithRawResponse",
    "UsageStatisticsResourceWithStreamingResponse",
    "AsyncUsageStatisticsResourceWithStreamingResponse",
    "ModelGroupsResource",
    "AsyncModelGroupsResource",
    "ModelGroupsResourceWithRawResponse",
    "AsyncModelGroupsResourceWithRawResponse",
    "ModelGroupsResourceWithStreamingResponse",
    "AsyncModelGroupsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import deployment_package_install_params, deployment_package_install_async_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.deployment_package_job import DeploymentPackageJob
from ..types.install_deployment_package_response import InstallDeploymentPackageResponse

__all__ = ["DeploymentPackagesResource", "AsyncDeploymentPackagesResource"]


class DeploymentPackagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DeploymentPackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return DeploymentPackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeploymentPackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return DeploymentPackagesResourceWithStreamingResponse(self)

    def install(
        self,
        account_id: str,
        *,
        deployment_package_id: str | NotGiven = NOT_GIVEN,
        deployment_package_name: str | NotGiven = NOT_GIVEN,
        deployment_package_type: Literal[
            "EGP_CONFIGURATION",
            "BLOB_STORAGE_FILE",
            "DOCKER_IMAGE",
            "EGP_ENTITY",
            "EGP_USER",
            "INTERNAL_MODEL_INFO",
            "EVALUATION_DATASET",
            "MODEL",
            "KNOWLEDGE_BASE",
        ]
        | NotGiven = NOT_GIVEN,
        version_constraint: deployment_package_install_params.VersionConstraint | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InstallDeploymentPackageResponse:
        """
        Install a deployment packages

        Args:
          deployment_package_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v4/deployment-packages/{account_id}/install",
            body=maybe_transform(
                {
                    "deployment_package_id": deployment_package_id,
                    "deployment_package_name": deployment_package_name,
                    "deployment_package_type": deployment_package_type,
                    "version_constraint": version_constraint,
                },
                deployment_package_install_params.DeploymentPackageInstallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InstallDeploymentPackageResponse,
        )

    def install_async(
        self,
        account_id: str,
        *,
        deployment_package_id: str | NotGiven = NOT_GIVEN,
        deployment_package_name: str | NotGiven = NOT_GIVEN,
        deployment_package_type: Literal[
            "EGP_CONFIGURATION",
            "BLOB_STORAGE_FILE",
            "DOCKER_IMAGE",
            "EGP_ENTITY",
            "EGP_USER",
            "INTERNAL_MODEL_INFO",
            "EVALUATION_DATASET",
            "MODEL",
            "KNOWLEDGE_BASE",
        ]
        | NotGiven = NOT_GIVEN,
        version_constraint: deployment_package_install_async_params.VersionConstraint | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeploymentPackageJob:
        """
        Install a deployment packages

        Args:
          deployment_package_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v4/deployment-packages/{account_id}/install_async",
            body=maybe_transform(
                {
                    "deployment_package_id": deployment_package_id,
                    "deployment_package_name": deployment_package_name,
                    "deployment_package_type": deployment_package_type,
                    "version_constraint": version_constraint,
                },
                deployment_package_install_async_params.DeploymentPackageInstallAsyncParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeploymentPackageJob,
        )


class AsyncDeploymentPackagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDeploymentPackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDeploymentPackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeploymentPackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncDeploymentPackagesResourceWithStreamingResponse(self)

    async def install(
        self,
        account_id: str,
        *,
        deployment_package_id: str | NotGiven = NOT_GIVEN,
        deployment_package_name: str | NotGiven = NOT_GIVEN,
        deployment_package_type: Literal[
            "EGP_CONFIGURATION",
            "BLOB_STORAGE_FILE",
            "DOCKER_IMAGE",
            "EGP_ENTITY",
            "EGP_USER",
            "INTERNAL_MODEL_INFO",
            "EVALUATION_DATASET",
            "MODEL",
            "KNOWLEDGE_BASE",
        ]
        | NotGiven = NOT_GIVEN,
        version_constraint: deployment_package_install_params.VersionConstraint | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InstallDeploymentPackageResponse:
        """
        Install a deployment packages

        Args:
          deployment_package_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v4/deployment-packages/{account_id}/install",
            body=await async_maybe_transform(
                {
                    "deployment_package_id": deployment_package_id,
                    "deployment_package_name": deployment_package_name,
                    "deployment_package_type": deployment_package_type,
                    "version_constraint": version_constraint,
                },
                deployment_package_install_params.DeploymentPackageInstallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InstallDeploymentPackageResponse,
        )

    async def install_async(
        self,
        account_id: str,
        *,
        deployment_package_id: str | NotGiven = NOT_GIVEN,
        deployment_package_name: str | NotGiven = NOT_GIVEN,
        deployment_package_type: Literal[
            "EGP_CONFIGURATION",
            "BLOB_STORAGE_FILE",
            "DOCKER_IMAGE",
            "EGP_ENTITY",
            "EGP_USER",
            "INTERNAL_MODEL_INFO",
            "EVALUATION_DATASET",
            "MODEL",
            "KNOWLEDGE_BASE",
        ]
        | NotGiven = NOT_GIVEN,
        version_constraint: deployment_package_install_async_params.VersionConstraint | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeploymentPackageJob:
        """
        Install a deployment packages

        Args:
          deployment_package_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v4/deployment-packages/{account_id}/install_async",
            body=await async_maybe_transform(
                {
                    "deployment_package_id": deployment_package_id,
                    "deployment_package_name": deployment_package_name,
                    "deployment_package_type": deployment_package_type,
                    "version_constraint": version_constraint,
                },
                deployment_package_install_async_params.DeploymentPackageInstallAsyncParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeploymentPackageJob,
        )


class DeploymentPackagesResourceWithRawResponse:
    def __init__(self, deployment_packages: DeploymentPackagesResource) -> None:
        self._deployment_packages = deployment_packages

        self.install = to_raw_response_wrapper(
            deployment_packages.install,
        )
        self.install_async = to_raw_response_wrapper(
            deployment_packages.install_async,
        )


class AsyncDeploymentPackagesResourceWithRawResponse:
    def __init__(self, deployment_packages: AsyncDeploymentPackagesResource) -> None:
        self._deployment_packages = deployment_packages

        self.install = async_to_raw_response_wrapper(
            deployment_packages.install,
        )
        self.install_async = async_to_raw_response_wrapper(
            deployment_packages.install_async,
        )


class DeploymentPackagesResourceWithStreamingResponse:
    def __init__(self, deployment_packages: DeploymentPackagesResource) -> None:
        self._deployment_packages = deployment_packages

        self.install = to_streamed_response_wrapper(
            deployment_packages.install,
        )
        self.install_async = to_streamed_response_wrapper(
            deployment_packages.install_async,
        )


class AsyncDeploymentPackagesResourceWithStreamingResponse:
    def __init__(self, deployment_packages: AsyncDeploymentPackagesResource) -> None:
        self._deployment_packages = deployment_packages

        self.install = async_to_streamed_response_wrapper(
            deployment_packages.install,
        )
        self.install_async = async_to_streamed_response_wrapper(
            deployment_packages.install_async,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options

__all__ = ["FeedbackResource", "AsyncFeedbackResource"]


class FeedbackResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FeedbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return FeedbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FeedbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return FeedbackResourceWithStreamingResponse(self)

    def delete(
        self,
        application_interaction_id: str,
        *,
        thread_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete Interaction Feedback

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        if not application_interaction_id:
            raise ValueError(
                f"Expected a non-empty value for `application_interaction_id` but received {application_interaction_id!r}"
            )
        return self._delete(
            f"/v4/threads/{thread_id}/messages/{application_interaction_id}/feedback",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncFeedbackResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFeedbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFeedbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFeedbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncFeedbackResourceWithStreamingResponse(self)

    async def delete(
        self,
        application_interaction_id: str,
        *,
        thread_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete Interaction Feedback

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        if not application_interaction_id:
            raise ValueError(
                f"Expected a non-empty value for `application_interaction_id` but received {application_interaction_id!r}"
            )
        return await self._delete(
            f"/v4/threads/{thread_id}/messages/{application_interaction_id}/feedback",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class FeedbackResourceWithRawResponse:
    def __init__(self, feedback: FeedbackResource) -> None:
        self._feedback = feedback

        self.delete = to_raw_response_wrapper(
            feedback.delete,
        )


class AsyncFeedbackResourceWithRawResponse:
    def __init__(self, feedback: AsyncFeedbackResource) -> None:
        self._feedback = feedback

        self.delete = async_to_raw_response_wrapper(
            feedback.delete,
        )


class FeedbackResourceWithStreamingResponse:
    def __init__(self, feedback: FeedbackResource) -> None:
        self._feedback = feedback

        self.delete = to_streamed_response_wrapper(
            feedback.delete,
        )


class AsyncFeedbackResourceWithStreamingResponse:
    def __init__(self, feedback: AsyncFeedbackResource) -> None:
        self._feedback = feedback

        self.delete = async_to_streamed_response_wrapper(
            feedback.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["KnowledgeBaseUploadFilesParams"]


class KnowledgeBaseUploadFilesParams(TypedDict, total=False):
    chunking_strategy_config: Required[str]

    data_source_config: Required[str]

    files: Required[List[FileTypes]]

    force_reupload: Required[bool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import FileTypes

__all__ = ["TrainingDatasetCreateParams"]


class TrainingDatasetCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    file: Required[FileTypes]
    """The file to upload as the training dataset"""

    name: Required[str]
    """The name of the dataset"""

    schema_type: Required[Literal["GENERATION", "RERANKING_QUESTIONS"]]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, TypeAlias, TypedDict

from .flexible_test_case_schema_param import FlexibleTestCaseSchemaParam
from .artifact_schema_generation_param import ArtifactSchemaGenerationParam
from .generation_test_case_schema_param import GenerationTestCaseSchemaParam

__all__ = ["TestCaseBatchParams", "Item", "ItemTestCaseData"]


class TestCaseBatchParams(TypedDict, total=False):
    items: Required[Iterable[Item]]


ItemTestCaseData: TypeAlias = Union[
    ArtifactSchemaGenerationParam, GenerationTestCaseSchemaParam, FlexibleTestCaseSchemaParam
]


class Item(TypedDict, total=False):
    test_case_data: Required[ItemTestCaseData]
    """The data for the test case in a format matching the provided schema_type"""

    account_id: str
    """The ID of the account that owns the given entity."""

    chat_history: object
    """Used for tracking previous chat interactions for multi-chat test cases"""

    test_case_metadata: object
    """Metadata for the test case"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from .chunks import (
    ChunksResource,
    AsyncChunksResource,
    ChunksResourceWithRawResponse,
    AsyncChunksResourceWithRawResponse,
    ChunksResourceWithStreamingResponse,
    AsyncChunksResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .themes import (
    ThemesResource,
    AsyncThemesResource,
    ThemesResourceWithRawResponse,
    AsyncThemesResourceWithRawResponse,
    ThemesResourceWithStreamingResponse,
    AsyncThemesResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .questions import (
    QuestionsResource,
    AsyncQuestionsResource,
    QuestionsResourceWithRawResponse,
    AsyncQuestionsResourceWithRawResponse,
    QuestionsResourceWithStreamingResponse,
    AsyncQuestionsResourceWithStreamingResponse,
)
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)
from .evaluations import (
    EvaluationsResource,
    AsyncEvaluationsResource,
    EvaluationsResourceWithRawResponse,
    AsyncEvaluationsResourceWithRawResponse,
    EvaluationsResourceWithStreamingResponse,
    AsyncEvaluationsResourceWithStreamingResponse,
)
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)
from .chat_threads import (
    ChatThreadsResource,
    AsyncChatThreadsResource,
    ChatThreadsResourceWithRawResponse,
    AsyncChatThreadsResourceWithRawResponse,
    ChatThreadsResourceWithStreamingResponse,
    AsyncChatThreadsResourceWithStreamingResponse,
)
from .interactions import (
    InteractionsResource,
    AsyncInteractionsResource,
    InteractionsResourceWithRawResponse,
    AsyncInteractionsResourceWithRawResponse,
    InteractionsResourceWithStreamingResponse,
    AsyncInteractionsResourceWithStreamingResponse,
)
from .model_groups import (
    ModelGroupsResource,
    AsyncModelGroupsResource,
    ModelGroupsResourceWithRawResponse,
    AsyncModelGroupsResourceWithRawResponse,
    ModelGroupsResourceWithStreamingResponse,
    AsyncModelGroupsResourceWithStreamingResponse,
)
from .question_sets import (
    QuestionSetsResource,
    AsyncQuestionSetsResource,
    QuestionSetsResourceWithRawResponse,
    AsyncQuestionSetsResourceWithRawResponse,
    QuestionSetsResourceWithStreamingResponse,
    AsyncQuestionSetsResourceWithStreamingResponse,
)
from .knowledge_bases import (
    KnowledgeBasesResource,
    AsyncKnowledgeBasesResource,
    KnowledgeBasesResourceWithRawResponse,
    AsyncKnowledgeBasesResourceWithRawResponse,
    KnowledgeBasesResourceWithStreamingResponse,
    AsyncKnowledgeBasesResourceWithStreamingResponse,
)
from .model_templates import (
    ModelTemplatesResource,
    AsyncModelTemplatesResource,
    ModelTemplatesResourceWithRawResponse,
    AsyncModelTemplatesResourceWithRawResponse,
    ModelTemplatesResourceWithStreamingResponse,
    AsyncModelTemplatesResourceWithStreamingResponse,
)
from .studio_projects import (
    StudioProjectsResource,
    AsyncStudioProjectsResource,
    StudioProjectsResourceWithRawResponse,
    AsyncStudioProjectsResourceWithRawResponse,
    StudioProjectsResourceWithStreamingResponse,
    AsyncStudioProjectsResourceWithStreamingResponse,
)
from .chat_completions import (
    ChatCompletionsResource,
    AsyncChatCompletionsResource,
    ChatCompletionsResourceWithRawResponse,
    AsyncChatCompletionsResourceWithRawResponse,
    ChatCompletionsResourceWithStreamingResponse,
    AsyncChatCompletionsResourceWithStreamingResponse,
)
from .fine_tuning_jobs import (
    FineTuningJobsResource,
    AsyncFineTuningJobsResource,
    FineTuningJobsResourceWithRawResponse,
    AsyncFineTuningJobsResourceWithRawResponse,
    FineTuningJobsResourceWithStreamingResponse,
    AsyncFineTuningJobsResourceWithStreamingResponse,
)
from .application_specs import (
    ApplicationSpecsResource,
    AsyncApplicationSpecsResource,
    ApplicationSpecsResourceWithRawResponse,
    AsyncApplicationSpecsResourceWithRawResponse,
    ApplicationSpecsResourceWithStreamingResponse,
    AsyncApplicationSpecsResourceWithStreamingResponse,
)
from .training_datasets import (
    TrainingDatasetsResource,
    AsyncTrainingDatasetsResource,
    TrainingDatasetsResourceWithRawResponse,
    AsyncTrainingDatasetsResourceWithRawResponse,
    TrainingDatasetsResourceWithStreamingResponse,
    AsyncTrainingDatasetsResourceWithStreamingResponse,
)
from .evaluation_configs import (
    EvaluationConfigsResource,
    AsyncEvaluationConfigsResource,
    EvaluationConfigsResourceWithRawResponse,
    AsyncEvaluationConfigsResourceWithRawResponse,
    EvaluationConfigsResourceWithStreamingResponse,
    AsyncEvaluationConfigsResourceWithStreamingResponse,
)
from .application_schemas import (
    ApplicationSchemasResource,
    AsyncApplicationSchemasResource,
    ApplicationSchemasResourceWithRawResponse,
    AsyncApplicationSchemasResourceWithRawResponse,
    ApplicationSchemasResourceWithStreamingResponse,
    AsyncApplicationSchemasResourceWithStreamingResponse,
)
from .application_threads import (
    ApplicationThreadsResource,
    AsyncApplicationThreadsResource,
    ApplicationThreadsResourceWithRawResponse,
    AsyncApplicationThreadsResourceWithRawResponse,
    ApplicationThreadsResourceWithStreamingResponse,
    AsyncApplicationThreadsResourceWithStreamingResponse,
)
from .deployment_packages import (
    DeploymentPackagesResource,
    AsyncDeploymentPackagesResource,
    DeploymentPackagesResourceWithRawResponse,
    AsyncDeploymentPackagesResourceWithRawResponse,
    DeploymentPackagesResourceWithStreamingResponse,
    AsyncDeploymentPackagesResourceWithStreamingResponse,
)
from .evaluation_datasets import (
    EvaluationDatasetsResource,
    AsyncEvaluationDatasetsResource,
    EvaluationDatasetsResourceWithRawResponse,
    AsyncEvaluationDatasetsResourceWithRawResponse,
    EvaluationDatasetsResourceWithStreamingResponse,
    AsyncEvaluationDatasetsResourceWithStreamingResponse,
)
from .application_variants import (
    ApplicationVariantsResource,
    AsyncApplicationVariantsResource,
    ApplicationVariantsResourceWithRawResponse,
    AsyncApplicationVariantsResourceWithRawResponse,
    ApplicationVariantsResourceWithStreamingResponse,
    AsyncApplicationVariantsResourceWithStreamingResponse,
)
from .application_deployments import (
    ApplicationDeploymentsResource,
    AsyncApplicationDeploymentsResource,
    ApplicationDeploymentsResourceWithRawResponse,
    AsyncApplicationDeploymentsResourceWithRawResponse,
    ApplicationDeploymentsResourceWithStreamingResponse,
    AsyncApplicationDeploymentsResourceWithStreamingResponse,
)
from .application_variant_reports import (
    ApplicationVariantReportsResource,
    AsyncApplicationVariantReportsResource,
    ApplicationVariantReportsResourceWithRawResponse,
    AsyncApplicationVariantReportsResourceWithRawResponse,
    ApplicationVariantReportsResourceWithStreamingResponse,
    AsyncApplicationVariantReportsResourceWithStreamingResponse,
)
from .knowledge_base_data_sources import (
    KnowledgeBaseDataSourcesResource,
    AsyncKnowledgeBaseDataSourcesResource,
    KnowledgeBaseDataSourcesResourceWithRawResponse,
    AsyncKnowledgeBaseDataSourcesResourceWithRawResponse,
    KnowledgeBaseDataSourcesResourceWithStreamingResponse,
    AsyncKnowledgeBaseDataSourcesResourceWithStreamingResponse,
)
from .application_test_case_outputs import (
    ApplicationTestCaseOutputsResource,
    AsyncApplicationTestCaseOutputsResource,
    ApplicationTestCaseOutputsResourceWithRawResponse,
    AsyncApplicationTestCaseOutputsResourceWithRawResponse,
    ApplicationTestCaseOutputsResourceWithStreamingResponse,
    AsyncApplicationTestCaseOutputsResourceWithStreamingResponse,
)

__all__ = [
    "KnowledgeBasesResource",
    "AsyncKnowledgeBasesResource",
    "KnowledgeBasesResourceWithRawResponse",
    "AsyncKnowledgeBasesResourceWithRawResponse",
    "KnowledgeBasesResourceWithStreamingResponse",
    "AsyncKnowledgeBasesResourceWithStreamingResponse",
    "KnowledgeBaseDataSourcesResource",
    "AsyncKnowledgeBaseDataSourcesResource",
    "KnowledgeBaseDataSourcesResourceWithRawResponse",
    "AsyncKnowledgeBaseDataSourcesResourceWithRawResponse",
    "KnowledgeBaseDataSourcesResourceWithStreamingResponse",
    "AsyncKnowledgeBaseDataSourcesResourceWithStreamingResponse",
    "ChunksResource",
    "AsyncChunksResource",
    "ChunksResourceWithRawResponse",
    "AsyncChunksResourceWithRawResponse",
    "ChunksResourceWithStreamingResponse",
    "AsyncChunksResourceWithStreamingResponse",
    "AgentsResource",
    "AsyncAgentsResource",
    "AgentsResourceWithRawResponse",
    "AsyncAgentsResourceWithRawResponse",
    "AgentsResourceWithStreamingResponse",
    "AsyncAgentsResourceWithStreamingResponse",
    "CompletionsResource",
    "AsyncCompletionsResource",
    "CompletionsResourceWithRawResponse",
    "AsyncCompletionsResourceWithRawResponse",
    "CompletionsResourceWithStreamingResponse",
    "AsyncCompletionsResourceWithStreamingResponse",
    "ChatCompletionsResource",
    "AsyncChatCompletionsResource",
    "ChatCompletionsResourceWithRawResponse",
    "AsyncChatCompletionsResourceWithRawResponse",
    "ChatCompletionsResourceWithStreamingResponse",
    "AsyncChatCompletionsResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "ModelGroupsResource",
    "AsyncModelGroupsResource",
    "ModelGroupsResourceWithRawResponse",
    "AsyncModelGroupsResourceWithRawResponse",
    "ModelGroupsResourceWithStreamingResponse",
    "AsyncModelGroupsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "QuestionSetsResource",
    "AsyncQuestionSetsResource",
    "QuestionSetsResourceWithRawResponse",
    "AsyncQuestionSetsResourceWithRawResponse",
    "QuestionSetsResourceWithStreamingResponse",
    "AsyncQuestionSetsResourceWithStreamingResponse",
    "EvaluationsResource",
    "AsyncEvaluationsResource",
    "EvaluationsResourceWithRawResponse",
    "AsyncEvaluationsResourceWithRawResponse",
    "EvaluationsResourceWithStreamingResponse",
    "AsyncEvaluationsResourceWithStreamingResponse",
    "EvaluationConfigsResource",
    "AsyncEvaluationConfigsResource",
    "EvaluationConfigsResourceWithRawResponse",
    "AsyncEvaluationConfigsResourceWithRawResponse",
    "EvaluationConfigsResourceWithStreamingResponse",
    "AsyncEvaluationConfigsResourceWithStreamingResponse",
    "EvaluationDatasetsResource",
    "AsyncEvaluationDatasetsResource",
    "EvaluationDatasetsResourceWithRawResponse",
    "AsyncEvaluationDatasetsResourceWithRawResponse",
    "EvaluationDatasetsResourceWithStreamingResponse",
    "AsyncEvaluationDatasetsResourceWithStreamingResponse",
    "StudioProjectsResource",
    "AsyncStudioProjectsResource",
    "StudioProjectsResourceWithRawResponse",
    "AsyncStudioProjectsResourceWithRawResponse",
    "StudioProjectsResourceWithStreamingResponse",
    "AsyncStudioProjectsResourceWithStreamingResponse",
    "ApplicationSpecsResource",
    "AsyncApplicationSpecsResource",
    "ApplicationSpecsResourceWithRawResponse",
    "AsyncApplicationSpecsResourceWithRawResponse",
    "ApplicationSpecsResourceWithStreamingResponse",
    "AsyncApplicationSpecsResourceWithStreamingResponse",
    "QuestionsResource",
    "AsyncQuestionsResource",
    "QuestionsResourceWithRawResponse",
    "AsyncQuestionsResourceWithRawResponse",
    "QuestionsResourceWithStreamingResponse",
    "AsyncQuestionsResourceWithStreamingResponse",
    "ModelTemplatesResource",
    "AsyncModelTemplatesResource",
    "ModelTemplatesResourceWithRawResponse",
    "AsyncModelTemplatesResourceWithRawResponse",
    "ModelTemplatesResourceWithStreamingResponse",
    "AsyncModelTemplatesResourceWithStreamingResponse",
    "FineTuningJobsResource",
    "AsyncFineTuningJobsResource",
    "FineTuningJobsResourceWithRawResponse",
    "AsyncFineTuningJobsResourceWithRawResponse",
    "FineTuningJobsResourceWithStreamingResponse",
    "AsyncFineTuningJobsResourceWithStreamingResponse",
    "TrainingDatasetsResource",
    "AsyncTrainingDatasetsResource",
    "TrainingDatasetsResourceWithRawResponse",
    "AsyncTrainingDatasetsResourceWithRawResponse",
    "TrainingDatasetsResourceWithStreamingResponse",
    "AsyncTrainingDatasetsResourceWithStreamingResponse",
    "DeploymentPackagesResource",
    "AsyncDeploymentPackagesResource",
    "DeploymentPackagesResourceWithRawResponse",
    "AsyncDeploymentPackagesResourceWithRawResponse",
    "DeploymentPackagesResourceWithStreamingResponse",
    "AsyncDeploymentPackagesResourceWithStreamingResponse",
    "ApplicationVariantsResource",
    "AsyncApplicationVariantsResource",
    "ApplicationVariantsResourceWithRawResponse",
    "AsyncApplicationVariantsResourceWithRawResponse",
    "ApplicationVariantsResourceWithStreamingResponse",
    "AsyncApplicationVariantsResourceWithStreamingResponse",
    "ApplicationDeploymentsResource",
    "AsyncApplicationDeploymentsResource",
    "ApplicationDeploymentsResourceWithRawResponse",
    "AsyncApplicationDeploymentsResourceWithRawResponse",
    "ApplicationDeploymentsResourceWithStreamingResponse",
    "AsyncApplicationDeploymentsResourceWithStreamingResponse",
    "ApplicationVariantReportsResource",
    "AsyncApplicationVariantReportsResource",
    "ApplicationVariantReportsResourceWithRawResponse",
    "AsyncApplicationVariantReportsResourceWithRawResponse",
    "ApplicationVariantReportsResourceWithStreamingResponse",
    "AsyncApplicationVariantReportsResourceWithStreamingResponse",
    "ApplicationTestCaseOutputsResource",
    "AsyncApplicationTestCaseOutputsResource",
    "ApplicationTestCaseOutputsResourceWithRawResponse",
    "AsyncApplicationTestCaseOutputsResourceWithRawResponse",
    "ApplicationTestCaseOutputsResourceWithStreamingResponse",
    "AsyncApplicationTestCaseOutputsResourceWithStreamingResponse",
    "ApplicationSchemasResource",
    "AsyncApplicationSchemasResource",
    "ApplicationSchemasResourceWithRawResponse",
    "AsyncApplicationSchemasResourceWithRawResponse",
    "ApplicationSchemasResourceWithStreamingResponse",
    "AsyncApplicationSchemasResourceWithStreamingResponse",
    "InteractionsResource",
    "AsyncInteractionsResource",
    "InteractionsResourceWithRawResponse",
    "AsyncInteractionsResourceWithRawResponse",
    "InteractionsResourceWithStreamingResponse",
    "AsyncInteractionsResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
    "ApplicationThreadsResource",
    "AsyncApplicationThreadsResource",
    "ApplicationThreadsResourceWithRawResponse",
    "AsyncApplicationThreadsResourceWithRawResponse",
    "ApplicationThreadsResourceWithStreamingResponse",
    "AsyncApplicationThreadsResourceWithStreamingResponse",
    "ChatThreadsResource",
    "AsyncChatThreadsResource",
    "ChatThreadsResourceWithRawResponse",
    "AsyncChatThreadsResourceWithRawResponse",
    "ChatThreadsResourceWithStreamingResponse",
    "AsyncChatThreadsResourceWithStreamingResponse",
    "ThemesResource",
    "AsyncThemesResource",
    "ThemesResourceWithRawResponse",
    "AsyncThemesResourceWithRawResponse",
    "ThemesResourceWithStreamingResponse",
    "AsyncThemesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ModelCreateParams", "BaseModelMetadata", "BaseModelMetadataModelDetails"]


class ModelCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    model_type: Required[Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"]]
    """An enum representing the different types of models supported.

    Attributes: COMPLETION: Denotes that the model type is completion.
    CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
    that the model type is agent. EMBEDDING: Denotes that the model type is
    embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
    that the model type is generic.
    """

    name: Required[str]

    base_model_id: str

    base_model_metadata: BaseModelMetadata

    description: str

    model_card: str

    model_creation_parameters: object

    model_group_id: str

    model_template_id: str

    model_vendor: Literal["OPENAI", "COHERE", "GOOGLE", "ANTHROPIC", "LAUNCH", "LLMENGINE", "BEDROCK", "OTHER"]
    """An enum representing the different types of model vendors supported.

    Attributes: OPENAI: Denotes that the model vendor is OpenAI. COHERE: Denotes
    that the model vendor is Cohere. GOOGLE: Denotes that the model vendor is
    Google. ANTHROPIC: Denotes that the model vendor is Anthropic. LLMENGINE:
    Denotes that the model vendor is LLM Engine. OTHER: Denotes that the model
    vendor is Other.
    """

    training_data_card: str


class BaseModelMetadataModelDetails(TypedDict, total=False):
    alignments: int

    languages: int

    number_of_parameters: int

    token_context_window: int


class BaseModelMetadata(TypedDict, total=False):
    delivery_date: str

    model_developer: str

    model_license_url: str

    model_details: Annotated[BaseModelMetadataModelDetails, PropertyInfo(alias="modelDetails")]

    ui_model_section_type: Literal["PARTNER", "OPENSOURCE", "CUSTOM"]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CustomChunkingStrategyConfig"]


class CustomChunkingStrategyConfig(BaseModel):
    endpoint: str
    """Endpoint path to call for custom chunking"""

    strategy: Literal["custom"]

    params: Optional[object] = None
    """Parameters that will be appended to the body of the request for the chunk."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TrainingDatasetGenerationItem"]


class TrainingDatasetGenerationItem(BaseModel):
    input: str

    output: str

    schema_type: Optional[Literal["GENERATION"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MessageUpdateResponse"]


class MessageUpdateResponse(BaseModel):
    id: str

    aggregated: bool

    application_spec_id: str

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    duration_ms: int
    """How much time the step took in milliseconds(ms)"""

    input: object

    operation_status: Literal["SUCCESS", "ERROR"]
    """An enumeration."""

    output: object

    start_timestamp: datetime

    chat_thread_id: Optional[str] = None

    interaction_source: Optional[Literal["EXTERNAL_AI", "EVALUATION", "SGP_CHAT"]] = None
    """An enumeration."""

    operation_metadata: Optional[object] = None
    """The JSON representation of the metadata insights emitted through the execution.

    This can differ based on different types of operations
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChunkExtraInfoSchema", "Chunk"]


class Chunk(TypedDict, total=False):
    metadata: Required[object]

    text: Required[str]


class ChunkExtraInfoSchema(TypedDict, total=False):
    chunks: Required[Iterable[Chunk]]

    schema_type: Literal["CHUNKS"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, TypedDict

__all__ = ["EvaluationListParams"]


class EvaluationListParams(TypedDict, total=False):
    account_id: str

    application_spec_id: Union[int, str]

    include_archived: bool

    limit: int
    """Maximum number of artifacts to be returned by the given endpoint.

    Defaults to 100 and cannot be greater than 10k.
    """

    page: int
    """Page number for pagination to be returned by the given endpoint.

    Starts at page 1
    """

    sort_by: List[
        Literal[
            "status:asc",
            "status:desc",
            "application_spec_id:asc",
            "application_spec_id:desc",
            "application_spec:asc",
            "application_spec:desc",
            "application_variant_id:asc",
            "application_variant_id:desc",
            "application_variant:asc",
            "application_variant:desc",
            "evaluation_config_id:asc",
            "evaluation_config_id:desc",
            "completed_at:asc",
            "completed_at:desc",
            "total_test_case_result_count:asc",
            "total_test_case_result_count:desc",
            "completed_test_case_result_count:asc",
            "completed_test_case_result_count:desc",
            "annotation_config:asc",
            "annotation_config:desc",
            "question_id_to_annotation_config:asc",
            "question_id_to_annotation_config:desc",
            "evaluation_config_expanded:asc",
            "evaluation_config_expanded:desc",
            "test_case_results:asc",
            "test_case_results:desc",
            "async_jobs:asc",
            "async_jobs:desc",
            "id:asc",
            "id:desc",
            "created_at:asc",
            "created_at:desc",
            "account_id:asc",
            "account_id:desc",
            "created_by_user_id:asc",
            "created_by_user_id:desc",
            "archived_at:asc",
            "archived_at:desc",
            "created_by_user:asc",
            "created_by_user:desc",
            "name:asc",
            "name:desc",
            "description:asc",
            "description:desc",
            "tags:asc",
            "tags:desc",
            "evaluation_config:asc",
            "evaluation_config:desc",
        ]
    ]

    view: List[Literal["ApplicationSpec", "AsyncJobs", "EvaluationConfig", "TestCaseResults"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import account_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.create_account_response import CreateAccountResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateAccountResponse:
        """
        Create a new account that with yourself as the admin.

        Args:
          account_name: The name of the account. This will be displayed in the Scale GenAI Platform UI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/accounts",
            body=maybe_transform({"account_name": account_name}, account_create_params.AccountCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateAccountResponse,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateAccountResponse:
        """
        Create a new account that with yourself as the admin.

        Args:
          account_name: The name of the account. This will be displayed in the Scale GenAI Platform UI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/accounts",
            body=await async_maybe_transform({"account_name": account_name}, account_create_params.AccountCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateAccountResponse,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.create = to_raw_response_wrapper(
            accounts.create,
        )


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.create = async_to_raw_response_wrapper(
            accounts.create,
        )


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.create = to_streamed_response_wrapper(
            accounts.create,
        )


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.create = async_to_streamed_response_wrapper(
            accounts.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = [
    "FlexibleMessage",
    "EgpAPIBackendServerInternalEntitiesUserMessage",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesTextUserMessageContentParts",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentParts",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentPartsImageURL",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentParts",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentPartsImageData",
    "EgpAPIBackendServerInternalEntitiesAssistantMessage",
    "EgpAPIBackendServerInternalEntitiesSystemMessage",
]


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesTextUserMessageContentParts(
    BaseModel
):
    text: str

    type: Optional[Literal["text"]] = None


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentPartsImageURL(
    BaseModel
):
    url: str
    """The URL of the image. Note: only OpenAI supports this."""

    detail: Optional[Literal["low", "high", "auto"]] = None
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentParts(
    BaseModel
):
    image_url: EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentPartsImageURL

    type: Optional[Literal["image_url"]] = None


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentPartsImageData(
    BaseModel
):
    data: str
    """The base64-encoded image data."""

    media_type: str
    """The media/mime type of the image data.

    For example, 'image/png'. Check providers' documentation for supported media
    types.
    """

    detail: Optional[Literal["low", "high", "auto"]] = None
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""

    type: Optional[Literal["base64"]] = None
    """The type of the image data. Only base64 is supported."""


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentParts(
    BaseModel
):
    image_data: EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentPartsImageData

    type: Optional[Literal["image_data"]] = None


EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1: TypeAlias = Annotated[
    Union[
        EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesTextUserMessageContentParts,
        EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentParts,
        EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentParts,
    ],
    PropertyInfo(discriminator="type"),
]


class EgpAPIBackendServerInternalEntitiesUserMessage(BaseModel):
    content: Union[str, List[EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1]]

    role: Optional[Literal["user"]] = None


class EgpAPIBackendServerInternalEntitiesAssistantMessage(BaseModel):
    content: str

    role: Optional[Literal["assistant"]] = None


class EgpAPIBackendServerInternalEntitiesSystemMessage(BaseModel):
    content: str

    role: Optional[Literal["system"]] = None


FlexibleMessage: TypeAlias = Annotated[
    Union[
        EgpAPIBackendServerInternalEntitiesUserMessage,
        EgpAPIBackendServerInternalEntitiesAssistantMessage,
        EgpAPIBackendServerInternalEntitiesSystemMessage,
    ],
    PropertyInfo(discriminator="role"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import chunk_rank_params, chunk_synthesis_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.ranked_chunks_response import RankedChunksResponse
from ..types.synthesize_chunks_response import SynthesizeChunksResponse

__all__ = ["ChunksResource", "AsyncChunksResource"]


class ChunksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChunksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ChunksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChunksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ChunksResourceWithStreamingResponse(self)

    def rank(
        self,
        *,
        query: str,
        rank_strategy: chunk_rank_params.RankStrategy,
        relevant_chunks: Iterable[chunk_rank_params.RelevantChunk],
        account_id: str | NotGiven = NOT_GIVEN,
        top_k: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RankedChunksResponse:
        """
        ### Description

        Sorts a list of text chunks by similarity against a given query string.

        ### Details

        Use this API endpoint to rank which text chunks provide the most relevant
        responses to a given a query string.

        This is useful for stuffing chunks into a prompt where order may matter or for
        filtering out less relevant chunks according to the ranking strategy. For
        example, this API may be useful when doing retrieval augment generation (RAG).
        Sometimes vector store
        [similarity search](https://scale-egp.readme.io/reference/query_vector_store)
        does not always return the best ranking of text chunks, since this is heavily
        dependent on embedding generation. This API endpoint can act as a
        post-processing step to re-sort the given chunks using more complex strategies
        that may outperform vector search, and then filter only the top-k most relevant
        chunks to stuff into the prompt for RAG.

        ### Restrictions and Limits

        Ranking can be a very intensive and slow process depending on methodology where
        duration scales with number of chunks. For best performance, we recommend
        ranking less than 640 chunks at a time, and you may see a decrease in
        performance as the number of chunks ranked increases.

        Args:
          query: Natural language query to re-rank chunks against. If a vector store query was
              originally used to retrieve these chunks, please use the same query for this
              ranking

          rank_strategy: The ranking strategy to use.

              Rank strategies determine how the ranking is done, They consist of the ranking
              method name and additional params needed to compute the ranking.

              Use the built-in `cross_encoder` or `rouge` strategies or create a custom one
              with the Models API.

          relevant_chunks: List of chunks to rank

          account_id: Account to rank chunks with. If you have access to more than one account, you
              must specify an account_id

          top_k: Number of chunks to return. Must be greater than 0 if specified. If not
              specified, all chunks will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/chunks/rank",
            body=maybe_transform(
                {
                    "query": query,
                    "rank_strategy": rank_strategy,
                    "relevant_chunks": relevant_chunks,
                    "account_id": account_id,
                    "top_k": top_k,
                },
                chunk_rank_params.ChunkRankParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RankedChunksResponse,
        )

    def synthesis(
        self,
        *,
        chunks: Iterable[chunk_synthesis_params.Chunk],
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SynthesizeChunksResponse:
        """
        ### Description

        Synthesizes a response to the given query using the given chunks as context.

        ### Details

        This API operates on "chunks," which are the result of
        [querying a vector store](https://scale-egp.readme.io/reference/query_vector_store).
        A chunk is simply a fragment of a larger document. It can optionally have its
        own metadata or ID. You can also construct your own chunks from scratch so long
        as you provide the text pertaining to each chunk.

        Synthesizing chunks can be thought of as the final step in a retrieval augmented
        generation ( RAG) system, after querying for chunks and potentially
        [ranking them](https://scale-egp.readme.io/reference/rank_chunks). Once you have
        the relevant chunks to use to respond to the user, you'll want to synthesize a
        readable natural language answer. During this synthesis step, we prompt an LLM
        with instructions and a set of (possibly transformed) chunks to guide it toward
        synthesizing a natural language response to the user query.

        There are several synthesis strategies, whose difference become more apparent as
        the number of chunks increases. Currently, EGP supports the `compact` strategy
        by default: stuff as many chunks into the synthesis LLM as possible, produce a
        best-effort answer, then continue to stuff and answer the next set of chunks.
        The answer is passed through and gradually refined with each iteration of chunk
        scanning. This continues until all chunks have been scanned and a final answer
        is synthesized. More synthesis strategies are currently in development!

        ### Restrictions and Limits

        Generally, chunk synthesis increases its token consumption linearly the number
        of chunk-tokens. The `compact` strategy attempts to be as token-efficient as
        possible by stuffing tokens into each round of chunk scanning. Other strategies
        (in development!) may produce better summaries or more precise answers at the
        expense of more tokens consumed.

        Broadly, we recommend keeping the number of chunks to under 100, or the total
        number of tokens across all chunks under around 10,000.

        Args:
          chunks: List of chunks to use to synthesize the response.

          query: Natural language query to resolve using the supplied chunks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/chunks/synthesis",
            body=maybe_transform(
                {
                    "chunks": chunks,
                    "query": query,
                },
                chunk_synthesis_params.ChunkSynthesisParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SynthesizeChunksResponse,
        )


class AsyncChunksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChunksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChunksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChunksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncChunksResourceWithStreamingResponse(self)

    async def rank(
        self,
        *,
        query: str,
        rank_strategy: chunk_rank_params.RankStrategy,
        relevant_chunks: Iterable[chunk_rank_params.RelevantChunk],
        account_id: str | NotGiven = NOT_GIVEN,
        top_k: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RankedChunksResponse:
        """
        ### Description

        Sorts a list of text chunks by similarity against a given query string.

        ### Details

        Use this API endpoint to rank which text chunks provide the most relevant
        responses to a given a query string.

        This is useful for stuffing chunks into a prompt where order may matter or for
        filtering out less relevant chunks according to the ranking strategy. For
        example, this API may be useful when doing retrieval augment generation (RAG).
        Sometimes vector store
        [similarity search](https://scale-egp.readme.io/reference/query_vector_store)
        does not always return the best ranking of text chunks, since this is heavily
        dependent on embedding generation. This API endpoint can act as a
        post-processing step to re-sort the given chunks using more complex strategies
        that may outperform vector search, and then filter only the top-k most relevant
        chunks to stuff into the prompt for RAG.

        ### Restrictions and Limits

        Ranking can be a very intensive and slow process depending on methodology where
        duration scales with number of chunks. For best performance, we recommend
        ranking less than 640 chunks at a time, and you may see a decrease in
        performance as the number of chunks ranked increases.

        Args:
          query: Natural language query to re-rank chunks against. If a vector store query was
              originally used to retrieve these chunks, please use the same query for this
              ranking

          rank_strategy: The ranking strategy to use.

              Rank strategies determine how the ranking is done, They consist of the ranking
              method name and additional params needed to compute the ranking.

              Use the built-in `cross_encoder` or `rouge` strategies or create a custom one
              with the Models API.

          relevant_chunks: List of chunks to rank

          account_id: Account to rank chunks with. If you have access to more than one account, you
              must specify an account_id

          top_k: Number of chunks to return. Must be greater than 0 if specified. If not
              specified, all chunks will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/chunks/rank",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "rank_strategy": rank_strategy,
                    "relevant_chunks": relevant_chunks,
                    "account_id": account_id,
                    "top_k": top_k,
                },
                chunk_rank_params.ChunkRankParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RankedChunksResponse,
        )

    async def synthesis(
        self,
        *,
        chunks: Iterable[chunk_synthesis_params.Chunk],
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SynthesizeChunksResponse:
        """
        ### Description

        Synthesizes a response to the given query using the given chunks as context.

        ### Details

        This API operates on "chunks," which are the result of
        [querying a vector store](https://scale-egp.readme.io/reference/query_vector_store).
        A chunk is simply a fragment of a larger document. It can optionally have its
        own metadata or ID. You can also construct your own chunks from scratch so long
        as you provide the text pertaining to each chunk.

        Synthesizing chunks can be thought of as the final step in a retrieval augmented
        generation ( RAG) system, after querying for chunks and potentially
        [ranking them](https://scale-egp.readme.io/reference/rank_chunks). Once you have
        the relevant chunks to use to respond to the user, you'll want to synthesize a
        readable natural language answer. During this synthesis step, we prompt an LLM
        with instructions and a set of (possibly transformed) chunks to guide it toward
        synthesizing a natural language response to the user query.

        There are several synthesis strategies, whose difference become more apparent as
        the number of chunks increases. Currently, EGP supports the `compact` strategy
        by default: stuff as many chunks into the synthesis LLM as possible, produce a
        best-effort answer, then continue to stuff and answer the next set of chunks.
        The answer is passed through and gradually refined with each iteration of chunk
        scanning. This continues until all chunks have been scanned and a final answer
        is synthesized. More synthesis strategies are currently in development!

        ### Restrictions and Limits

        Generally, chunk synthesis increases its token consumption linearly the number
        of chunk-tokens. The `compact` strategy attempts to be as token-efficient as
        possible by stuffing tokens into each round of chunk scanning. Other strategies
        (in development!) may produce better summaries or more precise answers at the
        expense of more tokens consumed.

        Broadly, we recommend keeping the number of chunks to under 100, or the total
        number of tokens across all chunks under around 10,000.

        Args:
          chunks: List of chunks to use to synthesize the response.

          query: Natural language query to resolve using the supplied chunks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/chunks/synthesis",
            body=await async_maybe_transform(
                {
                    "chunks": chunks,
                    "query": query,
                },
                chunk_synthesis_params.ChunkSynthesisParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SynthesizeChunksResponse,
        )


class ChunksResourceWithRawResponse:
    def __init__(self, chunks: ChunksResource) -> None:
        self._chunks = chunks

        self.rank = to_raw_response_wrapper(
            chunks.rank,
        )
        self.synthesis = to_raw_response_wrapper(
            chunks.synthesis,
        )


class AsyncChunksResourceWithRawResponse:
    def __init__(self, chunks: AsyncChunksResource) -> None:
        self._chunks = chunks

        self.rank = async_to_raw_response_wrapper(
            chunks.rank,
        )
        self.synthesis = async_to_raw_response_wrapper(
            chunks.synthesis,
        )


class ChunksResourceWithStreamingResponse:
    def __init__(self, chunks: ChunksResource) -> None:
        self._chunks = chunks

        self.rank = to_streamed_response_wrapper(
            chunks.rank,
        )
        self.synthesis = to_streamed_response_wrapper(
            chunks.synthesis,
        )


class AsyncChunksResourceWithStreamingResponse:
    def __init__(self, chunks: AsyncChunksResource) -> None:
        self._chunks = chunks

        self.rank = async_to_streamed_response_wrapper(
            chunks.rank,
        )
        self.synthesis = async_to_streamed_response_wrapper(
            chunks.synthesis,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ApplicationMetricScore", "LlmMetadata", "LlmMetadataUsage"]


class LlmMetadataUsage(BaseModel):
    completion_tokens: Optional[int] = None

    cost: Optional[float] = None

    model: Optional[Literal["gpt-4-turbo-2024-04-09", "gpt-3.5-turbo-0125", "gpt-4o-2024-05-13"]] = None
    """An enumeration."""

    prompt_tokens: Optional[int] = None


class LlmMetadata(BaseModel):
    logging: Optional[object] = None

    reasoning: Optional[str] = None

    time_elapsed_s: Optional[int] = None

    usage: Optional[List[LlmMetadataUsage]] = None


class ApplicationMetricScore(BaseModel):
    category: Literal["accuracy", "quality", "retrieval", "trust-and-safety"]
    """An enumeration."""

    metric_type: Literal[
        "answer-correctness",
        "answer-relevance",
        "faithfulness",
        "context-recall",
        "coherence",
        "grammar",
        "moderation",
        "safety",
        "safety-bias-and-stereotyping",
        "safety-opinions-disputed-topics",
        "safety-unethical-harmful-activities",
        "safety-copyright-violations",
        "safety-harmful-content",
        "safety-privacy-violations",
    ]

    llm_metadata: Optional[LlmMetadata] = None

    score: Optional[float] = None
